#!/usr/bin/env python
# -*- coding: euc-jp -*-

"""
  \file DataInPort.py
  \brief RTC::Port implementation for Data InPort
  \date $Date: 2007/09/20 $
  \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
 
  Copyright (C) 2006
      Noriaki Ando
      Task-intelligence Research Group,
      Intelligent Systems Research Institute,
      National Institute of
          Advanced Industrial Science and Technology (AIST), Japan
      All rights reserved.
"""
 
import OpenRTM
import RTC, RTC__POA


class DataInPort(OpenRTM.PortBase):
	
	"""
	\if jp
	\class DataInPort
	\brief InPort  Port
	\else
	\class DataInPort
	\brief InPort abstruct class
	\endif
	"""


	def __init__(self, name, inport):
		OpenRTM.PortBase.__init__(self, name)

		# PortProfile::properties 
		self.addProperty("port.port_type", "DataInPort")
		self._providers = []
		self._providers.append(OpenRTM.InPortCorbaProvider(inport))
		self._providers[-1].publishInterfaceProfile(self._profile.properties)
		self._consumers = []
		self._consumers.append(OpenRTM.OutPortCorbaConsumer(inport))
		self._dummy = [1]


	def __del__(self):
		pass


	###################################################################
	#                    Functor class                                #
	#                                                                 #
	class publish:
		
		def __init__(self, prop):
			self._prop = prop

		def __call__(self, provider):
			provider.publishInterface(self._prop)

	class subscribe:
		
		def __init__(self, prop):
			self._prop = prop

		def __call__(self, consumer):
			consumer.subscribeInterface(self._prop)

	class unsubscribe:
		
		def __init__(self, prop):
			self._prop = prop

		def __call__(self, consumer):
			consumer.unsubscribeInterface(self._prop)

	#                                                                 #
	###################################################################


	def publishInterfaces(self, connector_profile):
		if len(self._dummy) != 1:
			print "Memory access violation was detected."
			print "dummy.size(): ", len(self._dummy)
			print "size() should be 1."

		publish = self.publish(connector_profile.properties)
		for provider in self._providers:
			publish(provider)

		return RTC.RTC_OK
			

	def subscribeInterfaces(self, connector_profile):
		subscribe = self.subscribe(connector_profile.properties)
		for consumer in self._consumers:
			subscribe(consumer)

		return RTC.RTC_OK


	def unsubscribeInterfaces(self, connector_profile):
		unsubscribe = self.unsubscribe(connector_profile.properties)
		for consumer in self._consumers:
			unsubscribe(consumer)
		
    
