#!/usr/bin/env python
# -*- coding: euc-jp -*-

"""
 \file PortAdmin.py
 \brief RTC's Port administration class
 \date $Date: 2007/09/03 $
 \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara

 Copyright (C) 2006
     Task-intelligence Research Group,
     Intelligent Systems Research Institute,
     National Institute of
         Advanced Industrial Science and Technology (AIST), Japan
     All rights reserved.
"""



import RTC, RTC__POA
import OpenRTM


class PortAdmin:
	class comp_op:
		def __init__(self, name=None, factory=None):
			if name != None:
				self._name = name
			if factory != None:
				self._name = factory.getProfile().name

		def __call__(self, obj):
			name_ = obj.getProfile().name
			return self._name == name_
    

	class find_port_name:
		def __init__(self, name):
			self._name = name

		def __call__(self, p):
			prof = p.get_port_profile()
			name_ = prof.name 
			return self._name == name_


	class del_port:
		def __init__(self, pa):
			self._pa = pa

		def __call__(self, p):
			self._pa.deletePort(p)


	def __init__(self, orb, poa):
		# ORB ؤΥݥ
		self._orb = orb

		# POA ؤΥݥ
		self._poa = poa

		# PortΥ֥ȥե󥹤Υꥹ. PortList
		self._portRefs = []

		# ХȤľܳǼ륪֥ȥޥ͡
		self._portServants = OpenRTM.ObjectManager(self.comp_op)


	def getPortList(self):
		"""
		\if jp
		\brief PortList μ

		registerPort() ˤϿ줿 Port  PortList ؤΥݥ󥿤֤
		\return PortList PortList ؤΥݥ
		\else
		\brief Get PortList

		This operation returns the pointer to the PortList of Ports regsitered
		by registerPort().
		\return PortList+ The pointer points PortList
		\endif
		"""
		return self._portRefs


	def getPortRef(self, port_name):
		"""
		\if jp
		\brief Port Υ֥ȻȤμ

		port_name ǻꤷ Port Υ֥ȻȤ֤
		port_name ǻꤹ Port Ϥ餫 registerPort() ϿƤ
		ʤФʤʤ
		\param port_name Ȥ֤Port̾
		\return Port_ptr PortΥ֥Ȼ
		\else
		\brief Get PortList

		This operation returns the pointer to the PortList of Ports regsitered
		by registerPort().
		\param port_name The name of Port to be returned the reference.
		\return Port_ptr Port's object reference.
		\endif
		"""
		index = OpenRTM.CORBA_SeqUtil.find(self._portRefs, self.find_port_name(port_name))
		if index >= 0:
			return self._portRefs[index]
		return None


	def getPort(self, port_name):
		"""
		\if jp
		\brief Port ΥХȤΥݥ󥿤μ

		port_name ǻꤷ Port ΥХȤΥݥ󥿤֤
		port_name ǻꤹ Port Ϥ餫 registerPort() ϿƤ
		ʤФʤʤ
		\param port_name Ȥ֤Port̾
		\return PortBase PortХȴ쥯饹Υݥ
		\else
		\brief Getpointer to the Port's servant

		This operation returns the pointer to the PortBase servant regsitered
		by registerPort().
		\param port_name The name of Port to be returned the servant pointer.
		\return PortBase Port's servant's pointer.
		\endif
		"""
		return self._portServants.find(port_name)


	def registerPort(self, port):
		"""
		\if jp
		\brief Port Ͽ

		 port ǻꤵ줿 Port ΥХȤϿ롣
		Ͽ줿 Port ΥХȤϥ󥹥ȥ饯Ϳ줿POA 
		activate 졢Υ֥ȻȤPortProfile˥åȤ롣
		\param port Port Х
		\else
		\brief Regsiter Port

		This operation registers the Port's servant given by argument.
		The given Port's servant will be activated on the POA that is given
		to the constructor, and the created object reference is set
		to the Port's profile.
		\param port The Port's servant.
		\endif
		"""
		self._portRefs.append(port.getPortRef())
		self._portServants.registerObject(port)


	def deletePort(self, port):
		"""
		\if jp
		\brief Port Ͽ

		 port ǻꤵ줿 Port Ͽ롣
		 Port  deactivate 졢PortProfileΥե󥹤ˤϡ
		nilͤ롣
		\param port Port Х
		\else
		\brief Delete the Port's registration

		This operation unregisters the Port's registration.
		When the Port is unregistered, Port is deactivated, and the object
		reference in the Port's profile is set to nil.
		\param port The Port's servant.
		\endif
		"""
		try:
			port.disconnect_all()

			tmp = port.getProfile().name
			OpenRTM.CORBA_SeqUtil.erase_if(self._portRefs, self.find_port_name(tmp))

			self._poa.deactivate_object(self._pos.servant_to_id(port))
			port.setPotrRef(RTC.Port._nil)

			self._portServants.unregisterObject(tmp)
		except:
			pass


	def deletePortByName(self, port_name):
		"""
		\if jp
		\brief Port Ͽ

		ǻꤵ줿̾ Port Ͽ롣
		 Port  deactivate 졢PortProfileΥե󥹤ˤϡ
		nilͤ롣
		\param port_name Port ̾
		\else
		\brief Delete the Port' registration

		This operation delete the Port's registration specified by port_ name.
		When the Port is unregistered, Port is deactivated, and the object
		reference in the Port's profile is set to nil.
		\param port_name The Port's name.
		\endif
		"""
		if not port_name:
			return

		p = self._portServants.find(port_name)
		self.deletePort(p)


	def finalizePorts(self):
		"""
		\if jp
		\brief Ƥ Port deactivateϿ

		ϿƤƤPortФơХȤdeactivateԤ
		ϿꥹȤ롣
		\else
		\brief Unregister the Port

		This operation deactivates the all Port and deletes the all Port's
		registrations from the list.
		\endif
		"""
		ports = []
		ports = self._portServants.getObjects()
		len_ = len(ports)
		predi = self.del_port(self)
		for i in range(len_):
			idx = (len_ - 1) - i
			predi(ports[idx])
			


